import numpy as np
import random as rd
import unittest 

import sys

sys.path.insert(0, 'C:/Users/natji/statistical_methods/statistical_methods/')

from finance import *

class TestFinance(unittest.TestCase):

    def test_semivariance(self):
        assert True

if __name__ == '__main__':
    unittest.main()